module NodalData

!     External module parameters
      use Parameters, only : nRad, nTheta
      use Initiation, only : GridPoints

      implicit none

      private :: WriteToFile
      public :: FlowField


contains ! 2 subvortices


!=========================================================================
!                     Subroutine: StoreData
!=========================================================================
      subroutine FlowField(time)
!=========================================================================
!
!=========================================================================
!     Input
      real(kind=8), intent(in) :: time

!     Local variables
      character(len=100) :: fileName

!------------------------------ Executable Statements --------------------------

!     Write data to file
      write(fileName, 007) time
007   format( "Flow_t_=_", f10.5,".txt"  )
      call WriteToFile(11, trim(fileName), time)


      return
      end subroutine FlowField

!=========================================================================
!                     Subroutine: WriteToFile
!=========================================================================
      subroutine WriteToFile(iounit, fileName, time)
!=========================================================================
!
!=========================================================================
!     Input
      integer, intent(in) :: iounit
      real(kind=8), intent(in) :: time
      character(len=*), intent(in) :: fileName

!     Local variables
      integer :: ios, i, j

!------------------------------ Executable Statements --------------------------
!     Define output format
8001  format((F10.2),1x)
9001  format((ES20.12),1x)

!----------------------- Write nodal flow data to file ------------------------
!     Write the nodal data to a text file
!     Open file
      open(unit=iounit, file=fileName, iostat=ios, status="replace",&
       action="write", position="rewind")

        if ( ios /= 0 ) then
          print *, 'Error opening file:', fileName
          STOP
        end if

!       Title of file
        write(iounit,'(A)',iostat=ios, advance='NO') '# '
        write(iounit,'(A)',iostat=ios, advance='YES') fileName

!       Dimensionless time
        write(iounit,'(A)',iostat=ios, advance='NO') '# Ut/r : '
        write(iounit,8001,iostat=ios, advance='YES') time

!       Information about file
        write(iounit,'(A)',iostat=ios, advance='NO') '#         r         '
        write(iounit,'(A)',iostat=ios, advance='NO') '       theta        '
        write(iounit,'(A)',iostat=ios, advance='NO') '          Ux         '
        write(iounit,'(A)',iostat=ios, advance='NO') '          Uy        '
        write(iounit,'(A)',iostat=ios, advance='NO') '        Utot        '
        write(iounit,'(A)',iostat=ios, advance='NO') '          Psi         '
        write(iounit,'(A)',iostat=ios, advance='YES')'          d        '


!       Write data to file
        do j = 1, size(GridPoints, dim=2), 1
          do i = 1, size(GridPoints, dim=1), 1
!         r - coordinate
            write(iounit,9001,iostat=ios, advance='NO') &
            GridPoints(i,j,1)

!         theta - coordinate
            write(iounit,9001,iostat=ios, advance='NO') &
            GridPoints(i,j,3)

!         X - velocity
            write(iounit,9001,iostat=ios, advance='NO') &
            GridPoints(i,j,5)

!         Y - velocity
            write(iounit,9001,iostat=ios, advance='NO') &
            GridPoints(i,j,6)

!         Total velocity
            write(iounit,9001,iostat=ios, advance='NO') &
            sqrt(GridPoints(i,j,5)**2 + GridPoints(i,j,6)**2)

!         Stream function
            write(iounit,9001,iostat=ios, advance='NO')  GridPoints(i,j,7)

!         d
            write(iounit,9001,iostat=ios, advance='YES') GridPoints(i,j,4)


          end do
!         Last line equal to first line in each date group
!         r - coordinate
            write(iounit,9001,iostat=ios, advance='NO') &
            GridPoints(1,j,1)

!         theta - coordinate
            write(iounit,9001,iostat=ios, advance='NO') &
            GridPoints(1,j,3)

!         X - velocity
            write(iounit,9001,iostat=ios, advance='NO') &
            GridPoints(1,j,5)

!         Y - velocity
            write(iounit,9001,iostat=ios, advance='NO') &
            GridPoints(1,j,6)

!         Total velocity
            write(iounit,9001,iostat=ios, advance='NO') &
            sqrt(GridPoints(1,j,5)**2 + GridPoints(1,j,6)**2)

!         Stream function
            write(iounit,9001,iostat=ios, advance='NO')  GridPoints(1,j,7)

!         d
            write(iounit,9001,iostat=ios, advance='YES') GridPoints(1,j,4)

!         Introduce empty line
          write(iounit,*,iostat=ios) ' '

        end do

        if ( ios /= 0 ) then
          print '(A)', "Write error in file ", fileName
          stop
        end if

!     Close file
      close(unit = iounit, iostat=ios, status="keep")

      if ( ios /= 0 ) then
        print *, 'Error closing file:', fileName
        stop
      end if

      return
      end subroutine WriteToFile

end module NodalData
