module Parameters

      implicit none

      private :: readInput
      public  :: pi, e, tol, lenSequence, nTheta, nRad, nSub, nSuba, deltaT, &
                 rCylinder, rOuter, std, deltaAng, nu, thicknessFactor, uTheta,&
                 uRad, rho, nSubRed, nSubaRed, fs, timestep, Input, rCutoff,&
                 tCutoff

!     Constants
      real(kind=8), parameter :: pi = 3.1415926535897932d0
      real(kind=8), parameter :: e  = 2.7182818284590452d0

!     Parameters
      real(kind=8), parameter :: tol = 1.0d-10

!     Input
      integer :: lenSequence, nTheta, nRad, nSuba, nSub, nSubaRed, nSubRed, &
                 tCutoff
      real(kind=8) :: deltaT, rCylinder, rOuter, rCutoff, std, deltaAng, nu, &
                      rho, thicknessFactor, fs
      real(kind=8), allocatable, dimension(:) :: timestep
      real(kind=8), allocatable, dimension(:,:) :: uTheta, uRad

contains ! 2 subroutines

!===============================================================================
!                                Subroutine: Input
!===============================================================================
      subroutine Input
!===============================================================================
!
!
!===============================================================================

!     Local variables
      integer :: row, col
      real(kind=8), allocatable, dimension(:,:) :: dummyVariable

!------------------------------ Executable Statements --------------------------

!                                Integers variabels
!     nSub first time step
      nSub = 20

!     nSuba first time step
      nSuba = 10

!     nTheta
      call readInput(10,'Input.dat','nTheta', 1, 1, dummyVariable)
      nTheta = int(dummyVariable(1,1))

!     nRad
      call readInput(20,'Input.dat','nRad', 1, 1, dummyVariable)
      nRad = int(dummyVariable(1,1))

!     lenSequence
      call readInput(30,'Input.dat','lenSequence', 1, 1, dummyVariable)
      lenSequence = int(dummyVariable(1,1))

!     nSub after first time step
      call readInput(20,'Input.dat','nSub', 1, 1, dummyVariable)
      nSubRed = int(dummyVariable(1,1))

!     nSuba after first time step
      call readInput(30,'Input.dat','nSuba', 1, 1, dummyVariable)
      nSubaRed = int(dummyVariable(1,1))

!     tCutoff
      call readInput(30,'Input.dat','tCutoff', 1, 1, dummyVariable)
      tCutoff = int(dummyVariable(1,1))


!                             Double precision variabels
!     Time step
      call readInput(40,'Input.dat','dt', 1, 1,  dummyVariable)
      deltaT = dummyVariable(1,1)

!     Cylinder radius
      call readInput(50,'Input.dat','rCylinder', 1, 1,  dummyVariable)
      rCylinder = dummyVariable(1,1)

!     Domain size
      call readInput(50,'Input.dat','rOuter', 1, 1,  dummyVariable)
      rOuter = dummyVariable(1,1)

!     rCutoff
      call readInput(50,'Input.dat','rCutoff', 1, 1,  dummyVariable)
      rCutoff = dummyVariable(1,1)

!     Kinematic viscosity
      call readInput(60,'Input.dat','kinematic-viscosity', 1, 1,  dummyVariable)
      nu = dummyVariable(1,1)

!     Step size in the azimuthal direction
      call readInput(70,'Input.dat','deltaAng', 1, 1,  dummyVariable)
      deltaAng = dummyVariable(1,1)

!     Thickness factor of the first layer ( delta_r_1 = thicknessFactor * std )
      call readInput(80,'Input.dat','thicknessFactor', 1, 1,  dummyVariable)
      thicknessFactor = dummyVariable(1,1)

!     Standard deviation of diffuion process
      call readInput(90,'Input.dat','std', 1, 1,  dummyVariable)
      std = dummyVariable(1,1)

!     Standard deviation of diffuion process
      call readInput(100,'Input.dat','rho', 1, 1,  dummyVariable)
      rho = dummyVariable(1,1)

!     Frequence of storing data
      call readInput(120,'Input.dat','fs', 1, 1,  dummyVariable)
      fs = dummyVariable(1,1)


!                               Boundary conditions
!     Number of rows
      call readInput(130,'Tangential_velocity.txt','Rows:', 1, 1, dummyVariable)
      row = int(dummyVariable(1,1))

!     Number of columns
      call readInput(140,'Tangential_velocity.txt','Columns:', 1, 1,&
                     dummyVariable)
      col = int(dummyVariable(1,1)) + 1

!     Allocate memory
      allocate(uTheta((col-1), row), uRad((col-1), row), timestep(row))

!     Azimuthal velocity component at the boundary
      call readInput(150,'Tangential_velocity.txt','Time', row, col, &
                     dummyVariable)
      uTheta = transpose(dummyVariable(1:row,2:col))

!     Radial velocity component at the boundary
      call readInput(160,'Radial_velocity.txt','Time', row, col, dummyVariable)
      uRad = transpose(dummyVariable(1:row, 2:col))

!     Time steps (dimensionless time t/T)
      timestep = (dummyVariable(1:row, 1))

      return
      end subroutine Input
!

!===============================================================================
!                             Subroutine: readInput
!===============================================================================

subroutine readInput(iounit, fileName, variableName, row, col, output)
!===============================================================================
!     Opens the file "fileName" and search for the variable "variableName".
!     If the file contains the user spesified variable, the subroutine returns
!     the value/values of the variable as the double precision variable
!     "output"
!
!     Input:
!       iounit:
!       fileName:
!       variableName:
!       rows:
!       cols:
!
!     Returns
!       Output: Value/values of the user the variable "variableName" read from
!       the file named fileName
!===============================================================================

!     Input and output
      character(len=*), intent(in) :: fileName
      character(len=*), intent(in) :: variableName
      integer, intent(in) :: iounit, row, col
      double precision, intent(out), allocatable, dimension(:,:) :: output

!     Local variables
      logical :: Error = .true.
      integer :: pos, ios, i
      character(len=100) :: buffer, label

!------------------------------ Executable Statements --------------------------

!     Allocate memory
      if ( allocated(output) ) then
        if ( (size(output, dim=1) /= row) .or. &
             (size(output, dim=2) /= col) ) then
          deallocate(output)
          allocate(output(row,col))

        end if

      else
        allocate(output(row,col))

      end if

!     Open file
      open(unit=iounit, file=fileName, iostat=ios, status="old", action="read",&
      POSITION="rewind")

        if ( ios /= 0 ) then
          print *, 'Error opening file: ', fileName
          STOP
        end if

        if ( row == 1 .and. col == 1 ) then

!       Read from file
        do while ( ios == 0 )
!         Read line WITH SPACES INCLUDED
          read(iounit, '(a)', iostat = ios) buffer

          if ( ios == 0 ) then
!           Find the first instance of whitespace, then split label and data.
            pos = scan(buffer, '    ')
            label = buffer(1:pos)
            buffer = buffer(pos+1:)

            if (label == variableName) then
!             Store variable output
!             Read input from file
              read(buffer, *, iostat=ios) output
              Error = .false.
              exit ! Exit loop

            end if
          end if
        end do

        else

!       Read from file
        do while ( ios == 0 )
!         Read line WITH SPACES INCLUDED
          read(iounit, '(a)', iostat = ios) buffer

          if ( ios == 0 ) then
!           Find the first instance of whitespace, then split label and data.
            pos = scan(buffer, '    ')
            label = buffer(1:pos)
            buffer = buffer(pos+1:)

            if (label == variableName) then
!             Store variable output
!             Read input from file
              do i = 1, row, 1
                read(iounit, *, iostat=ios) output(i,:)
              end do
              Error = .false.
              exit ! Exit loop

            end if
          end if
        end do

      end if

      if ( Error ) then
        print *, 'Did not find ', variableName, ' in file ', fileName
      end if

!     Close file
      close(unit = iounit, iostat=ios, status="keep")
      if ( ios /= 0 ) then
        print *, 'Error closing file: ', fileName
        STOP
      end if

      return
      end subroutine readInput


end module Parameters
