program VIC

!     Modules
      use ForceAndPressure, only : Drag, Pressure, SurfaceVorticity, StoreVortex
      use HelpFunctions, only : intTOstr
      use Initiation, only : InitiationOfSystem, Head, Tail
      use LinkedListOperations, only : list_free, list_count
      use MatrixSolver, only : Solve
      use NodalData, only : FlowField
      use Parameters, only : Input, nSub, nSuba, nSubaRed, nSubRed, timestep
      use Random, only : Guassion
      use Velocity, only : VortexVelocity
      use Vorticity, only : NewVoricity, Projection, Reinject, VortexRemove
      use VorticityTransport, only : RandomWalk, Convetion

      use TypeDefine

      implicit none

!     Local variables
      integer :: nVortices, loop, loopCount
!------------------------------ Executable Statements --------------------------
!     Step one -  Initiation
      loopCount = 0

!     Read input from text files
      call Input

!     Initiate system
      call InitiationOfSystem

!     Create two random arrays of size lenSequence
      call Guassion

!     Step two - Enter main loop
      do loop = 1, size(timestep, dim=1), 1

        if ( mod(loop, 10) == 0 ) then

          if ( associated(Head) ) call list_count(Head, nVortices)

          print *, ""
          print *, ""
          print *, "Loop nr ", loop, " of ", size(timestep, dim=1)


          print *, ""
          print *, "Number of vortices: ", nVortices

        end if

!       Reduce number of subvortices
        if (loop  == 2) then
          nSub = nSubRed
          nSuba =  nSubaRed
        end if

!       Solve the Poisson equation and calculates the flow velocity at each
!       node
        call Solve(loop)

!       Create new vorticity at the body surface such that the no-slip condition
!       is satisfied
        call NewVoricity

        if( associated(Head) ) then

!         Calculate force coefficients
          call Drag(timestep(loop))

          !if ( c >= fs ) then
!           Calulate surface pressure
!            call Pressure(timestep(loop))

!           Calulate surface vorticity
!            call SurfaceVorticity(timestep(loop))

!           Store flow detales
!            call FlowField(timestep(loop))

!           Store the positon of the vortex particels
!4367  format( 'VortexParticles_at_t*_=_', f5.2'.txt' )
!            write(fileName, 4367) timestep(loop)
!            call StoreVortex(11, trim(fileName), timestep(loop))

          !  c = 0

          !else
          !  c = c + 1

          !end if

!         Calculate the flow velocities at each vortex particle
          call VortexVelocity

!         Solves the convection equation with the second order Runge-Kutta method
          call Convetion(loop)

!         Model the diffusion process by superimpose a random walk to the discrete
!         vortices
          call RandomWalk

!         Project the vorticity onto the nodes of the polar mesh
          call Projection

        end if

        if (loopCount >= 40) then
!         Removes all vortex particle
          call VortexRemove
          loopCount = 0

        else
          loopCount = loopCount + 1

        end if

      end do

!     Free the list
      call list_free(Head)

!     Deallocate and nullify the head and tail pointer
      nullify(Head)
      nullify(Tail)


end program VIC

!      print *, ""
!      print *, 'Uy'
!      do, i = 1, size(GridPoints, dim=1), 1
!          write(*,"(100g15.5)") ( GridPoints(i,j,6), j=1,size(GridPoints, dim=2), 1 )
!      enddo
